/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp org.freedesktop.ModemManager1.Modem.Time.xml -p Time -N
 *
 * qdbusxml2cpp is Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef TIME_H_1371820047
#define TIME_H_1371820047

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

#include "generic-types.h"

/*
 * Proxy class for interface org.freedesktop.ModemManager1.Modem.Time
 */
class OrgFreedesktopModemManager1ModemTimeInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.ModemManager1.Modem.Time"; }

public:
    OrgFreedesktopModemManager1ModemTimeInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = 0);

    ~OrgFreedesktopModemManager1ModemTimeInterface();

    Q_PROPERTY(QVariantMap NetworkTimezone READ networkTimezone)
    inline QVariantMap networkTimezone() const
    { return qvariant_cast< QVariantMap >(property("NetworkTimezone")); }

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<QString> GetNetworkTime()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QLatin1String("GetNetworkTime"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void NetworkTimeChanged(const QString &time);
};

#endif
