/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp org.freedesktop.ModemManager1.Sms.xml -p Sms -N
 *
 * qdbusxml2cpp is Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef SMS_H_1371819912
#define SMS_H_1371819912

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

#include "generic-types.h"

/*
 * Proxy class for interface org.freedesktop.ModemManager1.Sms
 */
class OrgFreedesktopModemManager1SmsInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.ModemManager1.Sms"; }

public:
    OrgFreedesktopModemManager1SmsInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = 0);

    ~OrgFreedesktopModemManager1SmsInterface();

    Q_PROPERTY(int SmsClass READ smsClass)
    inline int smsClass() const
    { return qvariant_cast< int >(property("Class")); }

    Q_PROPERTY(QByteArray Data READ data)
    inline QByteArray data() const
    { return qvariant_cast< QByteArray >(property("Data")); }

    Q_PROPERTY(bool DeliveryReportRequest READ deliveryReportRequest)
    inline bool deliveryReportRequest() const
    { return qvariant_cast< bool >(property("DeliveryReportRequest")); }

    Q_PROPERTY(uint DeliveryState READ deliveryState)
    inline uint deliveryState() const
    { return qvariant_cast< uint >(property("DeliveryState")); }

    Q_PROPERTY(QString DischargeTimestamp READ dischargeTimestamp)
    inline QString dischargeTimestamp() const
    { return qvariant_cast< QString >(property("DischargeTimestamp")); }

    Q_PROPERTY(uint MessageReference READ messageReference)
    inline uint messageReference() const
    { return qvariant_cast< uint >(property("MessageReference")); }

    Q_PROPERTY(QString Number READ number)
    inline QString number() const
    { return qvariant_cast< QString >(property("Number")); }

    Q_PROPERTY(uint PduType READ pduType)
    inline uint pduType() const
    { return qvariant_cast< uint >(property("PduType")); }

    Q_PROPERTY(QString SMSC READ sMSC)
    inline QString sMSC() const
    { return qvariant_cast< QString >(property("SMSC")); }

    Q_PROPERTY(uint State READ state)
    inline uint state() const
    { return qvariant_cast< uint >(property("State")); }

    Q_PROPERTY(uint Storage READ storage)
    inline uint storage() const
    { return qvariant_cast< uint >(property("Storage")); }

    Q_PROPERTY(QString Text READ text)
    inline QString text() const
    { return qvariant_cast< QString >(property("Text")); }

    Q_PROPERTY(QString Timestamp READ timestamp)
    inline QString timestamp() const
    { return qvariant_cast< QString >(property("Timestamp")); }

    Q_PROPERTY(ValidityPair Validity READ validity)
    inline ValidityPair validity() const
    { return qvariant_cast< ValidityPair >(property("Validity")); }

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> Send()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QLatin1String("Send"), argumentList);
    }

    inline QDBusPendingReply<> Store(uint storage)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(storage);
        return asyncCallWithArgumentList(QLatin1String("Store"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

#endif
